@@ -26,7 +26,7 @@ from io import BytesIO
 from paramiko.common import linefeed_byte, crlf, cr_byte
 from paramiko.file import BufferedFile
 
-from .util import needs_builtin
+from ._util import needs_builtin
 
 
 class LoopbackFile(BufferedFile):
@@ -127,8 +127,9 @@ class BufferedFileTest(unittest.TestCase):
         f.write("Not\nquite\n512 bytes.\n")
         self.assertEqual(f.read(1), b"")
         f.flush()
-        self.assertEqual(f.read(5), b"Not\nq")
-        self.assertEqual(f.read(10), b"uite\n512 b")
+        self.assertEqual(f.read(6), b"Not\nqu")
+        self.assertEqual(f.read(4), b"ite\n")
+        self.assertEqual(f.read(5), b"512 b")
         self.assertEqual(f.read(9), b"ytes.\n")
         self.assertEqual(f.read(3), b"")
         f.close()
